package patterns.behavioral.interpreter.client;

import patterns.behavioral.interpreter.Context;
import patterns.behavioral.interpreter.Expression;
import patterns.behavioral.interpreter.impl.BinaryExpression;
import patterns.behavioral.interpreter.impl.ConditionalExpression;
import patterns.behavioral.interpreter.impl.Constant;
import patterns.behavioral.interpreter.impl.Identifier;
import patterns.behavioral.interpreter.impl.TerminalExpression;

public class InterpreterClient {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String program = "if (a < 0) then 0 - a else a";
		Expression parseTree = parse(program);
		Context interpreterContext = new Context();
		
		// use a = -5
		System.out.println("setting a = -5");
		interpreterContext.assign("a", new Integer(-5));
		Object result = parseTree.interpret(interpreterContext);
		System.out.println("a = -5, (" + program + ") interprets to " + result);
		
		System.out.print("\n\n");
		
		System.out.println("setting a = 3");
		interpreterContext.assign("a", new Integer(3));
		result = parseTree.interpret(interpreterContext);
		System.out.println("a = 3, (" + program + ") interprets to " + result);
		
	}
	
	private static Expression parse(String program) {
		// TODO parse the program and get the parse tree.
		// for illustrating, we just hard-code the parse tree.
		// refer to builder and composite patter for how to 
		// parse and represent the parse tree
		
		// a
		TerminalExpression identifierA = new Identifier("a");
		// 0
		TerminalExpression constant0 = new Constant(new Integer(0));
		
		// a < 0
		BinaryExpression aLessThan0 = new BinaryExpression(identifierA, "<", constant0);
		
		// 0 - a
		BinaryExpression negativeA = new BinaryExpression(constant0, "-", identifierA);
		
		// if (a < 0) then 0 - a else a
		Expression conditionalExpression = new ConditionalExpression(aLessThan0, negativeA, identifierA);
		
		return conditionalExpression;
	}

}
