package patterns.behavioral.interpreter.impl;

import patterns.behavioral.interpreter.Context;
import patterns.behavioral.interpreter.Expression;

public class BinaryExpression extends Expression {

	private Expression left;
	private String operator;
	private Expression right;

	public BinaryExpression(Expression left, String operator,
			Expression right) {
		this.left = left;
		this.operator = operator;
		this.right = right;
	}

	@Override
	public Object interpret(Context context) {

		System.out.println("interpreting " + this.toString());

		// interpret LHS (left-hand-side) of the operator
		Object left = this.left.interpret(context);

		// interpret RHS expression of the operator;
		Object right = this.right.interpret(context);

		Object result;
		
		// apply the operator to get the value
		if (operator.equals("==")) { // operator ==
			result = left.equals(right);
		} 
		else if (operator.equals("!=")) { // operator !=
			result = !left.equals(right);
		} 
		else if (operator.equals("+")) { // operator +
			result = ((Integer) left).intValue() + ((Integer) right).intValue();
		}
		else if (operator.equals("-")) { // operator -
			result = ((Integer) left).intValue() - ((Integer) right).intValue();
		}
		else if (operator.equals("*")) { // operator *
			result = ((Integer) left).intValue() * ((Integer) right).intValue();
		}
		else if (operator.equals("/")) { // operator /
			return ((Integer) left).intValue() / ((Integer) right).intValue();
		}
		else if (operator.equals(">")) { // operator >
			result = ((Integer) left).intValue() > ((Integer) right).intValue();
		}
		else if (operator.equals("<")) { // operator <
			result = ((Integer) left).intValue() < ((Integer) right).intValue();
		} 
		else result = null;
		
		System.out.println(this.toString() + " interprets to " + result);
		
		return result;

	}

	public String toString() {
		return this.left.toString() + " " + this.operator + " " + this.right.toString();
		
	}
}
