package patterns.behavioral.interpreter.impl;

import patterns.behavioral.interpreter.Context;
import patterns.behavioral.interpreter.Expression;

public class ConditionalExpression extends Expression {

	private BinaryExpression conditionExpression;
	private Expression thenExpression;
	private Expression elseExpression;

	public ConditionalExpression(BinaryExpression conditionExpression,
			Expression thenExpression, Expression elseExpression) {
		this.conditionExpression = conditionExpression;
		this.thenExpression = thenExpression;
		this.elseExpression = elseExpression;
	}

	@Override
	public Object interpret(Context context) {
		// interpret the condition expression
		System.out.println("interpreting " + this.toString());
		
		Object booleanValue = conditionExpression.interpret(context);
		Object result = null;
		if (booleanValue.equals(Boolean.TRUE)) {
			// evaluates to true, interpret then part
			result = thenExpression.interpret(context);
		}
		else {
			if (elseExpression != null) {  // else part is optional
				// interpret the else part
				result = elseExpression.interpret(context);
			}
		}
		System.out.println(this.toString() + " interprets to " + result);
		return result;
	}
	
	public String toString() {
		return "if ( " + this.conditionExpression.toString() + " ) then " 
			+ this.thenExpression + " else " + this.elseExpression; 
	}
}
