package patterns.behavioral.memento.impl;

import patterns.behavioral.iterator.client.User;
import patterns.behavioral.memento.UndoableCommand;
import patterns.behavioral.memento.client.UserGenerator;

public class AddUserCommand extends UndoableCommand {
	
	private UserManager userManager;

	public void setUserManager(UserManager userManager) {
		this.userManager = userManager;
	}
	
	@Override
	public UndoableCommand clone() {
		AddUserCommand cloneResult = new AddUserCommand();
		// clone memento
		cloneResult.setUserManager(this.userManager);
		cloneResult.savedState = this.savedState;
		
		// TODO clone action-specific data
		
		return cloneResult;
	}	
	
	@Override
	public void execute() {
		User user = null;
		
		// TODO get user info from the user interface. Here, we use random-generated user
		user = UserGenerator.getRandomUser();
		
		// modify the model
		this.userManager.addUser(user);
		
		// create the memento to remember the added user
		this.savedState = userManager.createMemento();
	}

	@Override
	public void unexecute() {
		
		// get the memento that saves the added user
		UserCommandMemento memento = (UserCommandMemento) this.savedState;
		
		// remove the added user
		this.userManager.removeUser(memento.getIndex());

	}

	@Override
	public void reExecute() {
	
		// get the memento
		UserCommandMemento memento = (UserCommandMemento) this.savedState;
		
		// re-insert the removed user
		this.userManager.setMemento(this.savedState);
		
	}

}
