package patterns.behavioral.memento.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

import patterns.behavioral.memento.UndoableCommand;

public class CommandHistory {

	// the singleton instance
	private static CommandHistory instance = new CommandHistory();
	
	/**
	 * @return the singleton instance
	 */
	public static CommandHistory getInstance() {
		return instance;
	}

	// the command history
	private Stack<UndoableCommand> history = new Stack<UndoableCommand>();
	
	// the commands that are undone
	private Stack<UndoableCommand> undoHistory = new Stack<UndoableCommand>();
	
	
	private CommandHistory () {
	}
	
	// execute a new command
	public void addCommandToHistory(UndoableCommand command) {
		// if we are executing a new command, then the commands that are already undone are discarded
		this.undoHistory.clear();

		// add to the command history
		this.history.push(command.clone());
	}
	
	// undo a command
	public void undoCommand() {
		if (this.history.isEmpty()) {
			// no undo
			return;
		}
		
		// get the current command and undo it
		UndoableCommand command = this.history.pop();
		// undo the command
		command.unexecute();

		// save the undone command to the undo history
		this.undoHistory.push(command);
		
	}
	
	// redo a command
	public void redoCommand() {
		if (this.undoHistory.isEmpty()) {
			// no undone commands => no redo
			return;
		}
		
		// get the last undone command and redo it
		UndoableCommand command = this.undoHistory.pop();
		command.reExecute();
		
		// push it to command history
		this.history.push(command);
	}
	
}
