package patterns.behavioral.memento.impl;

import java.util.ArrayList;
import java.util.List;

import patterns.behavioral.iterator.client.User;
import patterns.behavioral.memento.Memento;
import patterns.behavioral.memento.Originator;

public class UserManager extends Originator {

	private List<User> managedUsers = new ArrayList<User>();
	
	private int currentIndex;
	
	int getCurrentIndex() {
		return currentIndex;
	}

	void setCurrentIndex(int currentIndex) {
		this.currentIndex = currentIndex;
	}

	public void addUser(User user) {
		System.out.println("adding user: " + user.toString());
		
		this.managedUsers.add(user);
		this.currentIndex = this.managedUsers.size() - 1;
	}
	
	public void setUser(User user, int atIndex) {
		System.out.println("setting user: " + user.toString() + " at index: " + atIndex);

		this.managedUsers.set(atIndex, user);
		this.currentIndex = atIndex;
	}
	
	public void removeUser(int atIndex) {
		System.out.println("removing user: " + this.managedUsers.get(atIndex).toString() + " at index: " + atIndex);
		
		this.managedUsers.remove(atIndex);
		this.currentIndex = atIndex;
	}
	
	public void setMemento(Memento memento) {
		UserCommandMemento state = (UserCommandMemento) memento;
		
		User userClone = state.getUser();
		int index = state.getIndex();
		
		// replace the user with the one from the memento
		if (this.managedUsers.size() > index)
			this.setUser(userClone, index);
		else if (this.managedUsers.size() == index)
			this.addUser(userClone);
		else
			throw new IllegalArgumentException("illegal memento applied.");
	}
	
	public Memento createMemento() {
		UserCommandMemento memento = new UserCommandMemento();
		memento.setIndex(this.currentIndex);
		
		User userClone = this.managedUsers.get(this.currentIndex).clone();
		memento.setUser(userClone);
		
		return memento;
	}
	
	private static UserManager instantce = new UserManager();
	
	public static UserManager getInstance() {
		return instantce;
	}
	
	
}
