package patterns.behavioral.state.impl;

import patterns.behavioral.state.Context;
import patterns.behavioral.state.State;

public class WanderingState extends State {

	public WanderingState(Context context) {
		super(context);
	}

	private void handleBattle() {
		boolean joinBattle = false;
		// check health point and determine if we want to join battle
		
		if (joinBattle) {
			// join battle
			this.context.setState(new InBattleState(this.context));
		}
		else {
			// escape. we cannot handle the enemy
			this.context.setState(new EscapingState(this.context));
		}
		
	}
	
	@Override
	public void handleApproached() {
		boolean isEnemy = false;
		// check whether the person approaching is enemy and assign to isEnemy
		
		if (isEnemy) { 
			// handle battle: join battle or escape
			this.handleBattle();
		}
		else {
			// start conversation
			this.context.setState(new InConversationState(this.context));
		}

	}

	@Override
	public void handleAttacked() {
		this.handleBattle();
	}

	@Override
	public void handleConversation() {
		// change to in conversation state
		this.context.setState(new InConversationState(this.context));
	}

	@Override
	public void handleLeft() {
		// do nothing. continue wandering
		
	}

}
