package patterns.behavioral.templatemethod;

import patterns.behavioral.templatemethod.impl.UserProfile;

public abstract class UserValidator {

	protected abstract boolean validateAddress(String address);
	
	protected abstract boolean validateAgeRestriction(int age);

	protected abstract boolean validatePhoneNumber(String phoneNumber);
	
	public boolean validate(UserProfile userProfile) {
		
		if (! this.validateName(userProfile.getName()))
			return false;  // failed name validation
		
		if (! this.validateAccountId(userProfile.getAccountId()))
			return false; // failed account ID validation
		
		if (! this.validatePhoneNumber(userProfile.getPhoneNumber()))
			return false; // failed phone number validation in subclass
		
		if (! this.validateAgeRestriction(userProfile.getAge()))
			return false; // failed age validation in subclass
		
		if (! this.validateAddress(userProfile.getAddress()))
			return false; // failed address validation in subclass
		
		return true;
		
	}
	
	private boolean validateName(String name) {
		// name should not be empty or white space only
		return name.trim().length() != 0;
	}
	
	private boolean validateAccountId(String accountId) {
		boolean accountExists = false;
		// check if there is already the account in the system

		return ! accountExists;
	}
}