package patterns.behavioral.visitor.impl;

import patterns.behavioral.visitor.Expression;
import patterns.behavioral.visitor.Visitor;

public class ConditionalExpression extends Expression {

	private BinaryExpression conditionExpression;
	private Expression thenExpression;
	private Expression elseExpression;

	public ConditionalExpression(BinaryExpression conditionExpression,
			Expression thenExpression, Expression elseExpression) {
		this.conditionExpression = conditionExpression;
		this.thenExpression = thenExpression;
		this.elseExpression = elseExpression;
	}

	@Override
	public Object accept(Visitor visitor) {
		
		// let visitor to visit self: calling visitConditionalExpression
		Object result = visitor.visitConditionalExpression(this);

		return result;
	}
	
	public BinaryExpression getConditionExpression() {
		return conditionExpression;
	}

	public Expression getThenExpression() {
		return thenExpression;
	}

	public Expression getElseExpression() {
		return elseExpression;
	}

	public String toString() {
		return "if ( " + this.conditionExpression.toString() + " ) then " 
			+ this.thenExpression + " else " + this.elseExpression; 
	}
}
