package patterns.behavioral.visitor.impl;

import java.util.HashMap;

import patterns.behavioral.visitor.Visitor;

public class InterpreterVisitor extends Visitor {

	// the context needed for during interpreting is stored in the visitor
	private HashMap<String, Object> valuesMap = new HashMap<String, Object>();
	
	public void assign(String identifier, Object value) {
		this.valuesMap.put(identifier, value);
	}
	
	
	@Override
	public Object visitBinaryExpression(BinaryExpression expression) {
		// interpret LHS (left-hand-side) of the operator
		Object left = expression.getLeft().accept(this);

		// interpret RHS expression of the operator;
		Object right = expression.getRight().accept(this);

		Object result = null;
		
		String operator = expression.getOperator();
		
		// apply the operator to get the value
		if (operator.equals("==")) { // operator ==
			result = left.equals(right);
		} 
		else if (operator.equals("!=")) { // operator !=
			result = !left.equals(right);
		} 
		else if (operator.equals("+")) { // operator +
			result = ((Integer) left).intValue() + ((Integer) right).intValue();
		}
		else if (operator.equals("-")) { // operator -
			result = ((Integer) left).intValue() - ((Integer) right).intValue();
		}
		else if (operator.equals("*")) { // operator *
			result = ((Integer) left).intValue() * ((Integer) right).intValue();
		}
		else if (operator.equals("/")) { // operator /
			return ((Integer) left).intValue() / ((Integer) right).intValue();
		}
		else if (operator.equals(">")) { // operator >
			result = ((Integer) left).intValue() > ((Integer) right).intValue();
		}
		else if (operator.equals("<")) { // operator <
			result = ((Integer) left).intValue() < ((Integer) right).intValue();
		} 
		else result = null;
		
		return result;
	}

	@Override
	public Object visitConditionalExpression(ConditionalExpression expression) {

		Object booleanValue = expression.getConditionExpression().accept(this);
		Object result = null;
		if (booleanValue.equals(Boolean.TRUE)) {
			// evaluates to true, interpret then part
			result = expression.getThenExpression().accept(this);
		}
		else {
			if (expression.getElseExpression() != null) {  // else part is optional
				// interpret the else part
				result = expression.getElseExpression().accept(this);
			}
		}
		
		return result;
	}

	@Override
	public Object visitConstant(Constant constant) {
		// constants interprets to their values
		return constant.getValue();
	}

	@Override
	public Object visitIdentifier(Identifier identifier) {
		// constants interprets to their value assignments
		return this.valuesMap.get(identifier.getName());
	
	}

}
