package patterns.behavioral.visitor.impl;

import patterns.behavioral.visitor.Expression;
import patterns.behavioral.visitor.Visitor;

public class X86CodeGenVisitor extends Visitor {

	@Override
	public Object visitBinaryExpression(BinaryExpression expression) {
		System.out.println("code generation for: " + expression.toString());
		
		Object result = null;  
		// TODO depending on the visitor operation, result may be assigned
		
		System.out.println("code generation for LHS: " + expression.getLeft().toString());
		// code generation for LHS
		Object left = expression.getLeft().accept(this);

		System.out.println("code generation for RHS: " + expression.getRight().toString());
		// code generation for RHS
		Object right = expression.getRight().accept(this);

		// TODO code generation for applying the operator
		System.out.println("code generation for applying the operator: " + expression.getOperator());
		
		System.out.println("finished code generation for: " + expression.toString());
		
		return result;
	}

	@Override
	public Object visitConditionalExpression(ConditionalExpression expression) {
		System.out.println("code generation for: " + expression.toString());

		Object result = null;  
		// TODO depending on the visitor operation, result may be assigned
		
		// visiting the condition part
		Expression condition = expression.getConditionExpression();
		System.out.println("code generation for condition: " + condition.toString() );
		condition.accept(this);
		
		// visiting the then part
		Expression thenExpression = expression.getThenExpression();
		System.out.println("code generation for then expression: " + thenExpression.toString() );
		thenExpression.accept(this);
		
		// visiting the else part
		Expression elseExpression = expression.getElseExpression();
		System.out.println("code generation for else expression: " + elseExpression.toString() );
		elseExpression.accept(this);
		
		System.out.println("finished code generation for: " + expression.toString());
		
		return result;
	}

	@Override
	public Object visitConstant(Constant constant) {
		System.out.println("code generation for: " + constant.toString());

		Object result = null;
		// TODO depending on the visitor operation, result may be assigned
		
		Object value = constant.getValue();
		// TODO code generation for the constant value
		
		System.out.println("finished code generation for: " + constant.toString());
		
		return result;
	}

	@Override
	public Object visitIdentifier(Identifier identifier) {
		System.out.println("code generation for: " + identifier.toString());

		Object result = null;
		// TODO depending on the visitor operation, result may be assigned
		
		Object name = identifier.getName();
		// TODO code generation for the identifier: address calculation, etc.
		
		System.out.println("finished code generation for: " + identifier.toString());
		
		return result;
	}

}
