package patterns.mvc;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JTextField;

public class SampleController_Mediator implements ActionListener {

	private JFrame mainWindow;
	private JButton addButton;
	private JButton deleteButton;
	private JList list;
	private JTextField textField;
	
	// serves as the data model
	private Vector<String> listData = new Vector<String>();
	
	public SampleController_Mediator() {
		this.mainWindow = new JFrame("Sample Window");
		this.mainWindow.setLayout(null);
		this.mainWindow.pack();
		this.mainWindow.setBounds(100, 100, 460, 520);
		this.textField = new JTextField();
		this.textField.setBounds(20, 20, 200, 20);
		this.mainWindow.add(this.textField);
		
		this.addButton = new JButton("Add");
		this.addButton.setBounds(240, 20, 80, 20);
		this.mainWindow.add(this.addButton);
		
		this.deleteButton = new JButton("Delete");
		this.deleteButton.setBounds(340, 20, 80, 20);
		this.mainWindow.add(this.deleteButton);
		
		this.list = new JList();
		this.list.setListData(this.listData);
		this.list.setBounds(20, 60, 400, 400);
		this.mainWindow.add(this.list);
		
		this.mainWindow.setVisible(true);
		
		this.setupListeners();
		
	}
	
	
	private void setupListeners() {
		// using named class
		this.mainWindow.addWindowListener(new MainWindowListener());
		
		// using anonymous class
		this.addButton.addActionListener(this);
		
		// using named class
		this.deleteButton.addActionListener(this);
		
	}


	public static void main(String [] args) {
		new SampleController_Mediator();
	}
	
	
	private class MainWindowListener implements WindowListener {

		public void windowActivated(WindowEvent e) { }
		public void windowClosed(WindowEvent e) { }
		public void windowClosing(WindowEvent e) {
			// only handle windowClosing event
			System.exit(0);
		}
		public void windowDeactivated(WindowEvent e) { }
		public void windowDeiconified(WindowEvent e) { }
		public void windowIconified(WindowEvent e) { }
		public void windowOpened(WindowEvent e) { }
		
	}
	
	// the method that via which widgets send notifications
	public void actionPerformed(ActionEvent e) {
		
		// determine which widget sends the event
		if (e.getSource() == this.addButton) {
			System.out.println("add button pushed");
			
			// get information from the text field
			String text = textField.getText();
			if (text.length() != 0) {
				System.out.println("add " + text + " to the list data");
				
				// modify the list data
				listData.add(text);
				
				// notify model changed
				list.updateUI();
			}
		}
		else if (e.getSource() == this.deleteButton) {
			System.out.println("delete button pushed");
			
			// change the data model
			int selectedIndex = list.getSelectedIndex();
				System.out.println("remove list data at index " + selectedIndex);
			
				if (selectedIndex < listData.size() && selectedIndex >= 0) {
				listData.remove(selectedIndex);
				
				// notify model changed
				list.updateUI();
			}
		}
		
	}
	
	
	
}
