package patterns.structural.decorator;

public class GamePortal {
	
	
	private static GamePortal instance;
	
	boolean cache;
	boolean log;
	boolean statistic;
	
	AccessController controller;


	public GamePortal(){

		controller = new RemoteAccessController();		
	}
	
	
	
	public void setCache(boolean cache) {
		
		if(cache){
			
			controller = new CacheDecorator(controller);
		
		}else{
			
			if(controller instanceof AccessControllerDecorator){
				controller = ((AccessControllerDecorator)controller).getOriginalController();
			}
		}
		
		this.cache = cache;
	}


	public void setLog(boolean log) {
		
		if(log){
			
			controller = new LogDecorator(controller);
		
		}else{
			
			if(controller instanceof AccessControllerDecorator){
				controller = ((AccessControllerDecorator)controller).getOriginalController();
			}
		}
		
		this.log = log;
	}


	public void setStatistic(boolean statistic) {
		
		if(statistic){
			
			controller = new StatisticDecorator(controller);
		
		}else{
			
			if(controller instanceof AccessControllerDecorator){
				controller = ((AccessControllerDecorator)controller).getOriginalController();
			}
		}
		
		this.statistic = statistic;
	}

	public void traffic(){
		login("Jeffrey","1234");
		login("Jeffrey","1234");
		login("Jeffrey","1234");
		
		login("Clement","5679");
		login("Clement","5678");
		login("Clement","5678");
	}
	
	public boolean login(String username,String password){
		
		boolean result = controller.authenticate(username, password);
			

		System.out.println("User ["+username+"] login "+
					(result ? "successfully" : "failed"));

		return result;
	}
	
	public static void main(String[] args){
		
		GamePortal portal = new GamePortal();
		
			
		System.out.println("Enable all decorator....\n");
		portal.setCache(true);
		portal.setStatistic(true);
		portal.setLog(true);		
		portal.traffic();
		
		System.out.println("\n\nDisable log and statistic decorator ....\n");
		portal.setLog(false);
		portal.setStatistic(false);		
		portal.traffic();
			
		System.out.println("\n\nEnable log decorator again....\n");
		portal.setLog(true);
		portal.traffic();
	}



	public static GamePortal getInstance() {
		if(instance == null)
			instance = new GamePortal();
		return instance;
	}
}
