package patterns.structural.facade;


import patterns.structural.bridge.*;
import patterns.structural.decorator.*;
import patterns.structural.composite.*;

public class StructuralPatternFacadeImpl implements StructuralPatternFacade{

	private static StructuralPatternFacade instance = null;
	
	private MemberManagementStore memberStore= null;
	private GamePortal gamePortal            = null;
	private TeamController teamController    = null;
	
	
	public StructuralPatternFacadeImpl(){
		
		memberStore = MemberManagementStore.getInstance();
		
		gamePortal = GamePortal.getInstance();
		gamePortal.setCache(true);
		gamePortal.setStatistic(true);
		gamePortal.setLog(true);
		
		teamController = new TeamController();
		
		// the team should already there, here we just pretend there was already one
		teamController.buildTeam();
		
	}

	public void broadcastMessage(String message) {
		
		teamController.broadcastMessage(message);
		
	}

	public void createMember(String username) {		
		memberStore.createMember(username);
	}

	public void login(String username, String password) {
		// TODO Auto-generated method stub
		
	}

	public static StructuralPatternFacade getInstance() {
		
		if(instance == null)
			instance = new StructuralPatternFacadeImpl();
		
		return instance;
	}
	
}
