package patterns.structural.proxy;

public class GamePortal {
	
	AccessController controller;
	
	public GamePortal(boolean cache){

		controller = new RemoteAccessController();
		
		if(cache)
			controller = new CacheProxy(controller);
		
	}
	
	
	public boolean login(String username,String password){
		
		return controller.authenticate(username, password);
	}
	
	public static void main(String[] args){
		
		GamePortal portal = new GamePortal(false);
	
		System.out.println("Without Cache Proxy....\n");
		
		System.out.println("Jeffrey's first attempt, login result = "+portal.login("Jeffrey","1234"));
		System.out.println("Jeffrey's second attempt, login result = "+portal.login("Jeffrey","1234"));
		System.out.println("Jeffrey's third attempt, login result = "+portal.login("Jeffrey","1234"));
		
		System.out.println("Clement's first attempt, login result = "+portal.login("Clement","5679"));
		System.out.println("Clement's second attempt, login result = "+portal.login("Clement","5678"));
		System.out.println("Clement's third attempt, login result = "+portal.login("Clement","5678"));
		
		
		System.out.println("\n\nWith Cache Proxy....\n");
		portal = new GamePortal(true);
		

		System.out.println("Jeffrey's first attempt, login result = "+portal.login("Jeffrey","1234"));
		System.out.println("Jeffrey's second attempt, login result = "+portal.login("Jeffrey","1234"));
		System.out.println("Jeffrey's third attempt, login result = "+portal.login("Jeffrey","1234"));
		
		System.out.println("Clement's first attempt, login result = "+portal.login("Clement","5679"));
		System.out.println("Clement's second attempt, login result = "+portal.login("Clement","5678"));
		System.out.println("Clement's third attempt, login result = "+portal.login("Clement","5678"));
	}
}
